"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Settings = void 0;

var _enums = require("../enums");

var _utils = require("../utils");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * A reader settings object with sensible defaults.
 */
class Settings {
  constructor() {
    _defineProperty(this, "appearance", _enums.Appearance.DEFAULT);

    _defineProperty(this, "fontFamily", _enums.FontFamily.ORIGINAL);

    _defineProperty(this, "textAlign", _enums.TextAlignment.JUSTIFY);

    _defineProperty(this, "colCount", _enums.ColumnCount.AUTO);

    _defineProperty(this, "scroll", false);

    _defineProperty(this, "fontOverride", false);

    _defineProperty(this, "verticalScroll", false);

    _defineProperty(this, "bodyHyphens", false);

    _defineProperty(this, "advancedSettings", true);

    _defineProperty(this, "fontSize", 100);

    _defineProperty(this, "wordSpacing", 0);

    _defineProperty(this, "letterSpacing", 0);

    _defineProperty(this, "pageMargins", 0);

    _defineProperty(this, "lineHeight", 1);

    _defineProperty(this, "paragraphMargins", 0);
  }

  static map(settings) {
    const defaultValues = new Settings();
    const mapped = {};
    Object.keys(defaultValues).forEach(key => {
      // @ts-ignore
      mapped[key] = settings[key] !== undefined ? settings[key] : defaultValues[key];
    });
    mapped.appearance = (0, _utils.indexOfObjectValue)(_enums.Appearance, mapped.appearance);
    mapped.fontFamily = (0, _utils.indexOfObjectValue)(_enums.FontFamily, mapped.fontFamily);
    mapped.textAlign = (0, _utils.indexOfObjectValue)(_enums.TextAlignment, mapped.textAlign);
    mapped.colCount = (0, _utils.indexOfObjectValue)(_enums.ColumnCount, mapped.colCount);
    Object.keys(_utils.RANGES).forEach(key => {
      // @ts-ignore
      mapped[key] = (0, _utils.clamp)(mapped[key], _utils.RANGES[key][0], _utils.RANGES[key][1]);
    });
    return mapped;
  }

}

exports.Settings = Settings;
//# sourceMappingURL=Settings.js.map