function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { Appearance, FontFamily, TextAlignment, ColumnCount } from '../enums';
import { RANGES, indexOfObjectValue, clamp } from '../utils';
/**
 * A reader settings object with sensible defaults.
 */

export class Settings {
  constructor() {
    _defineProperty(this, "appearance", Appearance.DEFAULT);

    _defineProperty(this, "fontFamily", FontFamily.ORIGINAL);

    _defineProperty(this, "textAlign", TextAlignment.JUSTIFY);

    _defineProperty(this, "colCount", ColumnCount.AUTO);

    _defineProperty(this, "scroll", false);

    _defineProperty(this, "fontOverride", false);

    _defineProperty(this, "verticalScroll", false);

    _defineProperty(this, "bodyHyphens", false);

    _defineProperty(this, "advancedSettings", true);

    _defineProperty(this, "fontSize", 100);

    _defineProperty(this, "wordSpacing", 0);

    _defineProperty(this, "letterSpacing", 0);

    _defineProperty(this, "pageMargins", 0);

    _defineProperty(this, "lineHeight", 1);

    _defineProperty(this, "paragraphMargins", 0);
  }

  static map(settings) {
    const defaultValues = new Settings();
    const mapped = {};
    Object.keys(defaultValues).forEach(key => {
      // @ts-ignore
      mapped[key] = settings[key] !== undefined ? settings[key] : defaultValues[key];
    });
    mapped.appearance = indexOfObjectValue(Appearance, mapped.appearance);
    mapped.fontFamily = indexOfObjectValue(FontFamily, mapped.fontFamily);
    mapped.textAlign = indexOfObjectValue(TextAlignment, mapped.textAlign);
    mapped.colCount = indexOfObjectValue(ColumnCount, mapped.colCount);
    Object.keys(RANGES).forEach(key => {
      // @ts-ignore
      mapped[key] = clamp(mapped[key], RANGES[key][0], RANGES[key][1]);
    });
    return mapped;
  }

}
//# sourceMappingURL=Settings.js.map